from django.conf import settings


def global_settings(request):
    return {
        'OTP_PRIMARY_COLOR': settings.OTP_PRIMARY_COLOR,
        'OTP_BACKGROUND_BTN': settings.OTP_BACKGROUND_BTN,
        'OTP_BACKGROUND_BTN_HOVER': settings.OTP_BACKGROUND_BTN_HOVER,
        'OTP_COLOR_TEXT_BTN': settings.OTP_COLOR_TEXT_BTN,
        'OTP_COLOR_TEXT_BTN_HOVER': settings.OTP_COLOR_TEXT_BTN_HOVER,
        'OTP_BRAND_NAME': settings.OTP_BRAND_NAME,
        'OTP_BRAND_IMG': settings.OTP_BRAND_IMG,
    }



from django.conf import settings


OTP_AUTH_PARAMS = getattr(settings, 'OTP_AUTH_PARAMS', [])
OTP_AUTH_TTL = getattr(settings, 'OTP_AUTH_TTL', 60 * 60 * 5)
OTP_AUTH_MESSAGE = getattr(
    settings, 'OTP_AUTH_MESSAGE', 'Utiliza {} como código de inicio de sesión.'
)
OTP_AUTH_COUNTRIES_CODES = getattr(settings, 'OTP_AUTH_COUNTRIES_CODES', [])
OTP_PRIMARY_COLOR = getattr(settings, 'OTP_PRIMARY_COLOR', '#002aff')
OTP_BACKGROUND_BTN = getattr(settings, 'OTP_BACKGROUND_BTN', '#002aff')
OTP_BACKGROUND_BTN_HOVER = getattr(settings, 'OTP_BACKGROUND_BTN_HOVER', '#ff7919')
OTP_COLOR_TEXT_BTN = getattr(settings, 'OTP_COLOR_TEXT_BTN', '#fff')
OTP_COLOR_TEXT_BTN_HOVER = getattr(settings, 'OTP_COLOR_TEXT_BTN_HOVER', '#fff')
OTP_BRAND_NAME = getattr(settings, 'OTP_BRAND_NAME', 'Axiacore')
OTP_BRAND_IMG = getattr(settings, 'OTP_BRAND_IMG', 'otp_auth/img/axiacore-logo.png')
OTP_CUSTOM_SMS_GATEWAY = getattr(settings, 'OTP_CUSTOM_SMS_GATEWAY', None)


AWS_ACCESS_KEY_ID = getattr(settings, 'OTP_AUTH_AWS_ACCESS_KEY_ID', '')
AWS_SECRET_ACCESS_KEY = getattr(settings, 'OTP_AUTH_AWS_SECRET_ACCESS_KEY', '')
AWS_DEFAULT_REGION = getattr(settings, 'OTP_AUTH_AWS_DEFAULT_REGION', 'us-east-1')



from urllib import parse
from secrets import token_urlsafe

from django.shortcuts import redirect
from django.urls import reverse
from django.views.generic import FormView, TemplateView

from .forms import StartForm, VerifyForm
from .hotp import HOTP
from .settings import OTP_AUTH_PARAMS


class StartView(FormView):
    template_name = 'otp_auth/start.html'
    form_class = StartForm

    def render_to_response(self, context, **response_kwargs):
        response = super().render_to_response(context, **response_kwargs)
        if not self.request.COOKIES.get('otp_unique_id', None):
            response.set_cookie('otp_unique_id', token_urlsafe())
        return response

    def form_valid(self, form):
        hotp = HOTP(unique_id=self.request.COOKIES['otp_unique_id'])
        hotp.create(
            country_code=form.cleaned_data['country_code'],
            phone_number=form.cleaned_data['phone_number']
        )

        params = {
            'country_code': form.cleaned_data['country_code'],
            'phone_number': form.cleaned_data['phone_number'],
            'redirect': self.request.GET['redirect'],
        }
        if OTP_AUTH_PARAMS:
            for param in OTP_AUTH_PARAMS:
                params[param] = self.request.GET.get(param)

        return redirect('{}?{}'.format(
            reverse('otp_auth:verify'),
            parse.urlencode(params, safe='/'))
        )


class VerifyView(FormView):
    template_name = 'otp_auth/verify.html'
    form_class = VerifyForm

    def get_initial(self):
        initial = super().get_initial()
        initial.update({
            'country_code': self.request.GET.get('country_code'),
            'phone_number': self.request.GET.get('phone_number'),
        })
        return initial

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        params = {'redirect': self.request.GET['redirect']}
        if OTP_AUTH_PARAMS:
            for param in OTP_AUTH_PARAMS:
                params[param] = self.request.GET.get(param)
        context['sms_url'] = '{}?{}'.format(
            reverse('otp_auth:start'),
            parse.urlencode(params, safe='/')
        )
        return context

    def form_valid(self, form):
        cleaned_data = form.cleaned_data

        hotp = HOTP(unique_id=self.request.COOKIES.get('otp_unique_id', ''))

        token = hotp.verify(cleaned_data.get('code', ''), phone_number=cleaned_data['phone_number'])
        if not token:
            form.add_error('code', 'Código no es valido')
            return self.form_invalid(form)

        params = {'token': token, 'redirect': self.request.GET['redirect']}
        if OTP_AUTH_PARAMS:
            for param in OTP_AUTH_PARAMS:
                params[param] = self.request.GET.get(param)

        return redirect('{}?{}'.format(reverse('otp_auth:done'), parse.urlencode(params, safe='/')))


class DoneView(TemplateView):
    template_name = 'otp_auth/done.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        params = {'token': self.request.GET['token']}
        if OTP_AUTH_PARAMS:
            for param in OTP_AUTH_PARAMS:
                params[param] = self.request.GET.get(param)

        context['redirect'] = '{}?{}'.format(
            self.request.GET['redirect'],
            parse.urlencode(params, safe='/')
        )
        return context



from django.urls import path

from . import views


app_name = 'ax3_otp_auth'
urlpatterns = [
    path(
        'start/',
        views.StartView.as_view(),
        name='start',
    ),

    path(
        'verify/',
        views.VerifyView.as_view(),
        name='verify',
    ),

    path(
        'done/',
        views.DoneView.as_view(),
        name='done',
    ),
]



COLOMBIA = 57
ARGENTINA = 54
BOLIVIA = 591
CHILE = 56
COSTA_RICA = 506
CUBA = 53
DOMINICAN_REPUBLIC = 809
ECUADOR = 593
GUATEMALA = 502
MEXICO = 52
PERU = 51

CONTRY_CODE_CHOICES = (
    (COLOMBIA, f'+{COLOMBIA} - Colombia'),
    (ARGENTINA, f'+{ARGENTINA} - Argentina'),
    (BOLIVIA, f'+{BOLIVIA} - Bolivia'),
    (CHILE, f'+{CHILE} - Chile'),
    (COSTA_RICA, f'+{COSTA_RICA} - Costa rica'),
    (CUBA, f'+{CUBA} - Cuba'),
    (DOMINICAN_REPUBLIC, f'+{DOMINICAN_REPUBLIC} - Dominican republic'),
    (ECUADOR, f'+{ECUADOR} - Ecuador'),
    (GUATEMALA, f'+{GUATEMALA} - Guatemala'),
    (MEXICO, f'+{MEXICO} - México'),
    (PERU, f'+{PERU} - Perú'),
)



from secrets import token_urlsafe

from django.core.cache import cache
from django.utils.module_loading import import_string

import boto3
import pyotp

from . import settings


class HOTP:
    def __init__(self, unique_id: str, digits: int = 6):
        self._unique_id = unique_id
        self._digits = digits
        self._ttl = settings.OTP_AUTH_TTL

    def _create_secret(self, secret: str) -> str:
        cache.set('{}.secret'.format(self._unique_id), secret, timeout=self._ttl)
        return secret

    def _create_counter(self) -> str:
        try:
            cache.incr('{}.counter'.format(self._unique_id))
        except ValueError:
            cache.set('{}.counter'.format(self._unique_id), 1, timeout=self._ttl)
        return cache.get('{}.counter'.format(self._unique_id))

    def _create_token(self, phone_number: int) -> str:
        token = token_urlsafe()
        cache.set(token, phone_number, timeout=self._ttl)
        return token

    def _get_secret(self):
        return cache.get('{}.secret'.format(self._unique_id))

    def _get_counter(self):
        return cache.get('{}.counter'.format(self._unique_id))

    def _send_sms(self, sms_code: int, country_code: str, phone_number: int):
        message = settings.OTP_AUTH_MESSAGE.format(sms_code)

        if settings.OTP_CUSTOM_SMS_GATEWAY:
            gateway = import_string(settings.OTP_CUSTOM_SMS_GATEWAY)
            gateway(country_code=country_code, phone_number=phone_number, message=message)

        else:
            sns = boto3.client(
                'sns',
                aws_access_key_id=settings.AWS_ACCESS_KEY_ID,
                aws_secret_access_key=settings.AWS_SECRET_ACCESS_KEY,
                region_name=settings.AWS_DEFAULT_REGION
            )

            sns.publish(
                PhoneNumber=f'+{country_code}{phone_number}',
                Message=message,
                MessageAttributes={
                    'AWS.SNS.SMS.SMSType': {
                        'DataType': 'String',
                        'StringValue': 'Transactional'
                    }
                }
            )

    def create(self, country_code: str, phone_number: int):
        secret = self._create_secret(secret=pyotp.random_base32(length=32))
        counter = self._create_counter()

        hotp = pyotp.HOTP(secret, digits=self._digits)
        self._send_sms(
            sms_code=hotp.at(counter),
            country_code=country_code,
            phone_number=phone_number
        )

    def verify(self, sms_code: int, phone_number: int) -> str:
        secret = self._get_secret()
        count = self._get_counter()

        if count and secret:
            hotp = pyotp.HOTP(secret, digits=self._digits)
            if hotp.verify(sms_code, count):
                return self._create_token(phone_number=phone_number)
        return None

    def get_phone_number(self, token: str) -> int:
        phone_number = cache.get(token)
        cache.delete(token)
        cache.delete_pattern('{}.*'.format(self._unique_id))
        return phone_number



from django.contrib.auth.backends import ModelBackend, UserModel


class OTPAuthBackend(ModelBackend):
    # pylint: disable=arguments-differ
    def authenticate(self, request, mobile_phone=None):
        try:
            user = UserModel.objects.get(mobile_phone=mobile_phone)
        except (UserModel.DoesNotExist, UserModel.MultipleObjectsReturned):
            return None
        else:
            if self.user_can_authenticate(user):
                return user
        return None



from django import forms
from django.core.validators import MinLengthValidator, RegexValidator

from .data import CONTRY_CODE_CHOICES
from .settings import OTP_AUTH_COUNTRIES_CODES


class StartForm(forms.Form):
    country_code = forms.ChoiceField(
        choices=CONTRY_CODE_CHOICES,
    )

    phone_number = forms.CharField(
        max_length=25,
        validators=[
            RegexValidator(r'^[0-9]*$', message='Ingresa solo números'),
            MinLengthValidator(10),
        ]
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        country_codes = []
        for code, label in CONTRY_CODE_CHOICES:
            if code in OTP_AUTH_COUNTRIES_CODES:
                country_codes.append((code, label))
        self.fields['country_code'].choices = country_codes


class VerifyForm(StartForm, forms.Form):
    HIDDEN_FIELDS = ['country_code', 'phone_number']

    code = forms.CharField(
        max_length=6,
        validators=[
            RegexValidator(r'^[0-9]*$', message='Ingresa solo números'),
            MinLengthValidator(6),
        ]
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        for field in self.HIDDEN_FIELDS:
            self.fields[field].widget = forms.HiddenInput()



import os
from setuptools import find_packages, setup

__VERSION__ = '1.0.1'

with open('README.md', 'r') as fh:
    long_description = fh.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='AX3 OTP Auth',
    version=__VERSION__,
    packages=find_packages(),
    include_package_data=True,
    description='AX3 OTP Auth is a very simple Django library for generating and verifying one-time passwords using HTOP guidelines.',
    long_description_content_type='text/markdown',
    long_description=long_description,
    url='https://github.com/Axiacore/ax3-OTP-Auth',
    author='Axiacore',
    author_email='info@axiacore.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.2',
        'Intended Audience :: Developers',
        "License :: OSI Approved :: MIT License",
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=[
        'pyotp >= 2.3.0',
        'django >= 2.2.0',
        'boto3 >= 1.9.244'
    ],
)
